/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.htmlcheck.dict;

import com.silverwrist.venice.htmlcheck.ModSpellingDictionary;

public class TreeLexicon implements ModSpellingDictionary
{
  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private DictNode root;    // root node of the trie that contains our data
  private int count;        // number of words in the lexicon

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public TreeLexicon()
  {
    root = new DictNode();
    count = 0;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Implementations from interface SpellingDictionary
   *--------------------------------------------------------------------------------
   */

  public synchronized int size()
  {
    return count;

  } // end size

  public synchronized boolean checkWord(String word)
  {
    String real_word = word.toLowerCase();
    DictNode cur = root;
    for (int i=0; i<real_word.length(); i++)
    { // loop through the characters and move down the trie
      char ch = real_word.charAt(i);
      if (((ch<'a') || (ch>'z')) && (ch!='-'))
	return false;
      cur = cur.getRef(ch);
      if (cur==null)
	return false;

    } // end for

    return cur.isTerminal();

  } // end checkWord

  /*--------------------------------------------------------------------------------
   * Implementations from interface ModSpellingDictionary
   *--------------------------------------------------------------------------------
   */

  public synchronized void addWord(String word)
  {
    String real_word = word.toLowerCase();
    DictNode cur = root;
    for (int i=0; i<real_word.length(); i++)
    { // move through the word - bail out if bogus chars
      char ch = real_word.charAt(i);
      if (((ch<'a') || (ch>'z')) && (ch!='-'))
	return;

      // advance down through trie
      DictNode next = cur.getRef(ch);
      if (next==null)
      { // strike out into a new DictNode
	next = new DictNode();
	cur.setRef(ch,next);

      } // end if

      cur = next;

    } // end for

    cur.setTerminal();
    count++;

  } // end addWord

  public void delWord(String word)
  { // not implemented for this lexicon type
  } // end delWord

  public synchronized void clear()
  {
    root = new DictNode();
    count = 0;

  } // end clear

} // end class TreeLexicon


